"""

m2wsgi.__main__:  allow m2wsgi to be executed directly by python -m
===================================================================

This is a simple script that calls the m2wsgi.main() function.

"""

import m2wsgi

if __name__ == "__main__":
    m2wsgi.main()

